package com.link.core.parserplugin.parser;

import com.link.core.parserplugin.AbstractScriptParser;
import com.link.core.parserplugin.util.ParserUtil;
import java.lang.reflect.Method;
import java.util.Iterator;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import org.springframework.expression.Expression;
import org.springframework.expression.ExpressionParser;
import org.springframework.expression.spel.standard.SpelExpressionParser;
import org.springframework.expression.spel.support.StandardEvaluationContext;

public class SpringELParser extends AbstractScriptParser {
   private static final String POUND = "#";
   private static final String apostrophe = "'";
   private final ExpressionParser parser = new SpelExpressionParser();
   private final ConcurrentHashMap<String, Expression> expCache = new ConcurrentHashMap();
   private static Method hash = null;
   private static Method empty = null;
   private final ConcurrentHashMap<String, Method> funcs = new ConcurrentHashMap(8);

   public void addFunction(String name, Method method) {
      this.funcs.put(name, method);
   }

   public <T> T getElValue(String keySpEL, Object target, Object[] arguments, Object retVal, boolean hasRetVal, Object userVal, boolean hasUserVal, Class<T> valueType) throws Exception {
      if (valueType.equals(String.class) && keySpEL.indexOf("#") == -1 && keySpEL.indexOf("'") == -1) {
         return keySpEL;
      } else {
         StandardEvaluationContext context = new StandardEvaluationContext();
         context.registerFunction("hash", hash);
         context.registerFunction("empty", empty);
         Iterator<Map.Entry<String, Method>> it = this.funcs.entrySet().iterator();

         while(it.hasNext()) {
            Map.Entry<String, Method> entry = (Map.Entry)it.next();
            context.registerFunction((String)entry.getKey(), (Method)entry.getValue());
         }

         context.setVariable("target", target);
         context.setVariable("args", arguments);
         if (hasUserVal) {
            context.setVariable("userVal", userVal);
         }

         if (hasRetVal) {
            context.setVariable("retVal", retVal);
         }

         Expression expression = (Expression)this.expCache.get(keySpEL);
         if (null == expression) {
            expression = this.parser.parseExpression(keySpEL);
            this.expCache.put(keySpEL, expression);
         }

         return expression.getValue(context, valueType);
      }
   }

   static {
      try {
         hash = ParserUtil.class.getDeclaredMethod("getUniqueHashStr", Object.class);
         empty = ParserUtil.class.getDeclaredMethod("isEmpty", Object.class);
      } catch (NoSuchMethodException var1) {
         NoSuchMethodException e = var1;
         e.printStackTrace();
      } catch (SecurityException var2) {
         SecurityException e = var2;
         e.printStackTrace();
      } catch (Exception var3) {
         Exception e = var3;
         e.printStackTrace();
      }

   }
}
